@@ -15,10 +15,22 @@ struct fdnode_st {
     int fd;
     int events;
     int fde_ndx;
+  #ifdef _WIN32
+    int fda_ndx;
+  #endif
 };
 
 /* These must match POLL* values from operating system headers */
 
+#ifdef _WIN32 /* MS is different; definitely not better */
+#define FDEVENT_IN     (0x0100 | 0x0200)
+#define FDEVENT_PRI    (0x0400)
+#define FDEVENT_OUT    (0x0010)
+#define FDEVENT_ERR    (0x0001)
+#define FDEVENT_HUP    (0x0002)
+#define FDEVENT_NVAL   (0x0004)
+#define FDEVENT_RDHUP  0x2000
+#else
 #define FDEVENT_IN     0x0001
 #define FDEVENT_PRI    0x0002
 #define FDEVENT_OUT    0x0004
@@ -30,6 +42,7 @@ struct fdnode_st {
 #else
 #define FDEVENT_RDHUP  0x2000
 #endif
+#endif
 
 #define FDEVENT_STREAM_REQUEST                  BV(0)
 #define FDEVENT_STREAM_REQUEST_BUFMIN           BV(1)
@@ -83,6 +96,8 @@ int fdevent_fcntl_set_nb_cloexec(int fd);
 int fdevent_fcntl_set_nb_cloexec_sock(int fd);
 int fdevent_socket_cloexec(int domain, int type, int protocol);
 int fdevent_socket_nb_cloexec(int domain, int type, int protocol);
+int fdevent_socketpair_cloexec(int domain, int typ, int protocol, int sv[2]);
+int fdevent_socketpair_nb_cloexec(int domain, int typ, int protocol, int sv[2]);
 int fdevent_dup_cloexec(int fd);
 int fdevent_open_cloexec(const char *pathname, int symlinks, int flags, mode_t mode);
 int fdevent_pipe_cloexec (int *fds, unsigned int bufsz_hint);
@@ -97,11 +112,39 @@ char ** fdevent_environ(void);
 
 int fdevent_open_devnull(void);
 int fdevent_open_dirname(char *path, int symlinks);
+#ifndef _WIN32
 int fdevent_set_stdin_stdout_stderr(int fdin, int fdout, int fderr);
 pid_t fdevent_fork_execve(const char *name, char *argv[], char *envp[], int fdin, int fdout, int fderr, int dfd);
-int fdevent_waitpid(pid_t pid, int *status, int nb);
-int fdevent_waitpid_intr(pid_t pid, int *status);
+#endif
+pid_t fdevent_waitpid(pid_t pid, int *status, int nb);
+pid_t fdevent_waitpid_intr(pid_t pid, int *status);
+int fdevent_kill(pid_t pid, int sig);
 
+#ifdef _WIN32
+__attribute_cold__
+void fdevent_win32_cleanup (void);
+
+#define fdevent_fork_execve(name, argv, envp, fdin, fdout, fderr, dfd) \
+        fdevent_createprocess((argv),(envp),(fdin),(fdout),(fderr),(dfd))
+pid_t fdevent_createprocess(char *argv[], char *envp[], intptr_t fdin, intptr_t fdout, int fderr, int dfd);
+#endif /* _WIN32 */
+
+#define fdio_close_dirfd(fd) close(fd)
+#define fdio_close_file(fd) close(fd)
+#define fdio_close_pipe(fd) close(fd)
+#define fdio_close_socket(fd) fdevent_socket_close(fd)
+#ifdef _WIN32
+int fdevent_socket_set_cloexec(int fd);
+int fdevent_socket_clr_cloexec(int fd);
+int fdevent_socket_set_nb(int fd);
+int fdevent_socket_set_nb_cloexec(int fd);
+#else
+#define fdevent_socket_set_cloexec(fd)    (fdevent_setfd_cloexec(fd), 0)
+#define fdevent_socket_clr_cloexec(fd)    (fdevent_clrfd_cloexec(fd), 0)
+#define fdevent_socket_set_nb(fd)         fdevent_fcntl_set_nb(fd)
+#define fdevent_socket_set_nb_cloexec(fd) fdevent_fcntl_set_nb_cloexec(fd)
+#endif
+int fdevent_socket_close (int fd);
 ssize_t fdevent_socket_read_discard (int fd, char *buf, size_t sz, int family, int so_type);
 
 int fdevent_ioctl_fionread (int fd, int fdfmt, int *toread);
