@@ -531,6 +531,18 @@ static handler_t mod_auth_require_parse_array(const array *value, array * const
 				return HANDLER_ERROR;
 			}
 
+			for (uint32_t o = 0; o < n; ++o) {
+				const buffer *k = &((data_array *)value->data[o])->key;
+				if (buffer_clen(&da_file->key) >= buffer_clen(k)
+				    && 0 == strncmp(da_file->key.ptr, k->ptr, buffer_clen(k))) {
+					log_error(errh, __FILE__, __LINE__,
+					  "auth.require path (\"%s\") will never match due to "
+					  "earlier match (\"%s\"); fix by sorting longer paths "
+					  "before shorter paths", da_file->key.ptr, k->ptr);
+					break;
+				}
+			}
+
 			if (require) { /*(always true at this point)*/
 				data_auth * const dauth = data_auth_init();
 				buffer_copy_buffer(&dauth->key, &da_file->key);
@@ -692,6 +704,7 @@ static handler_t mod_auth_uri_handler(request_st * const r, void *p_d) {
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_auth_plugin_init(plugin *p);
 int mod_auth_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
@@ -849,7 +862,7 @@ mod_auth_check_basic(request_st * const r, void *p_d, const struct http_auth_req
     default:
         log_error(r->conf.errh, __FILE__, __LINE__,
           "password doesn't match for %s username: %s IP: %s",
-          r->uri.path.ptr, user, r->con->dst_addr_buf.ptr);
+          r->uri.path.ptr, user, r->dst_addr_buf->ptr);
         r->keep_alive = -1; /*(disable keep-alive if bad password)*/
         rc = mod_auth_send_401_unauthorized_basic(r, require->realm);
         break;
@@ -1420,7 +1433,7 @@ mod_auth_digest_validate_params (request_st * const r, const struct http_auth_re
         log_error(r->conf.errh, __FILE__, __LINE__,
           "digest: auth failed: uri mismatch (%s != %.*s), IP: %s",
           r->target_orig.ptr, (int)dp->len[e_uri], dp->ptr[e_uri],
-          r->con->dst_addr_buf.ptr);
+          r->dst_addr_buf->ptr);
         return mod_auth_send_400_bad_request(r);
     }
 
@@ -1533,7 +1546,7 @@ mod_auth_check_digest (request_st * const r, void *p_d, const struct http_auth_r
         /* digest not ok */
         log_error(r->conf.errh, __FILE__, __LINE__,
           "digest: auth failed for %.*s: wrong password, IP: %s",
-          (int)ai.ulen, ai.username, r->con->dst_addr_buf.ptr);
+          (int)ai.ulen, ai.username, r->dst_addr_buf->ptr);
         r->keep_alive = -1; /*(disable keep-alive if bad password)*/
         return mod_auth_send_401_unauthorized_digest(r, require, 0);
     }
