@@ -11,7 +11,10 @@
 
 #include "first.h"
 
+#include "sys-dirent.h"
+#include "sys-stat.h"
 #include "sys-time.h"
+#include "sys-unistd.h" /* <unistd.h> */
 
 #include "base.h"
 #include "log.h"
@@ -30,9 +33,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
-#include <dirent.h>
 #include <fcntl.h>
-#include <unistd.h>
 
 #ifdef AT_FDCWD
 #ifndef _ATFILE_SOURCE
@@ -40,12 +41,10 @@
 #endif
 #endif
 
-#ifndef _D_EXACT_NAMLEN
-#ifdef _DIRENT_HAVE_D_NAMLEN
-#define _D_EXACT_NAMLEN(d) ((d)->d_namlen)
-#else
-#define _D_EXACT_NAMLEN(d) (strlen ((d)->d_name))
-#endif
+#ifdef _WIN32
+#include <windows.h>
+#include <stringapiset.h>
+#include <stdio.h>      /* FILENAME_MAX */
 #endif
 
 /**
@@ -117,7 +116,9 @@ typedef struct {
 #define DIRLIST_BLOB_SIZE      16
 
 typedef struct {
+  #ifndef _WIN32
 	DIR *dp;
+  #endif
 	dirls_list_t dirs;
 	dirls_list_t files;
 	char *path;
@@ -130,6 +131,11 @@ typedef struct {
 	char *jfn;
 	uint32_t jfn_len;
 	plugin_config conf;
+  #ifdef _WIN32
+	HANDLE hFind;
+	WIN32_FIND_DATAW ffd;
+	char fnUTF8[FILENAME_MAX*4+1];
+  #endif
 } handler_ctx;
 
 #define DIRLIST_BATCH 32
@@ -138,13 +144,21 @@ static int dirlist_max_in_progress;
 
 static handler_ctx * mod_dirlisting_handler_ctx_init (plugin_data * const p) {
     handler_ctx *hctx = ck_calloc(1, sizeof(*hctx));
+  #ifdef _WIN32
+    hctx->hFind = INVALID_HANDLE_VALUE;
+  #endif
     memcpy(&hctx->conf, &p->conf, sizeof(plugin_config));
     return hctx;
 }
 
 static void mod_dirlisting_handler_ctx_free (handler_ctx *hctx) {
+  #ifdef _WIN32
+    if (INVALID_HANDLE_VALUE != hctx->hFind)
+        FindClose(hctx->hFind);
+  #else
     if (hctx->dp)
         closedir(hctx->dp);
+  #endif
     if (hctx->files.ent) {
         dirls_entry_t ** const ent = hctx->files.ent;
         for (uint32_t i = 0, used = hctx->files.used; i < used; ++i)
@@ -159,12 +173,12 @@ static void mod_dirlisting_handler_ctx_free (handler_ctx *hctx) {
     }
     if (hctx->jb) {
         chunk_buffer_release(hctx->jb);
+        if (-1 != hctx->jfd)
+            close(hctx->jfd);
         if (hctx->jfn) {
             unlink(hctx->jfn);
             free(hctx->jfn);
         }
-        if (-1 != hctx->jfd)
-            close(hctx->jfd);
     }
     free(hctx->path);
     free(hctx);
@@ -986,8 +1000,8 @@ static void http_list_directory_footer(request_st * const r, const handler_ctx *
 
 static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
     const uint32_t dlen = buffer_clen(&r->physical.path);
-#if defined __WIN32
-    hctx->name_max = FILENAME_MAX;
+#ifdef _WIN32
+    hctx->name_max = FILENAME_MAX*4; /*(260 chars * 4 for (max) UTF-8 bytes)*/
 #else
 #ifndef PATH_MAX
 #define PATH_MAX 4096
@@ -997,10 +1011,30 @@ static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
 #endif
     hctx->path = ck_malloc(dlen + hctx->name_max + 1);
     memcpy(hctx->path, r->physical.path.ptr, dlen+1);
-  #if defined(HAVE_XATTR) || defined(HAVE_EXTATTR) || !defined(_ATFILE_SOURCE)
+  #if defined(HAVE_XATTR) || defined(HAVE_EXTATTR) \
+   || (!defined(_ATFILE_SOURCE) && !defined(_WIN32))
     hctx->path_file = hctx->path + dlen;
   #endif
 
+  #ifdef _WIN32
+    hctx->path[dlen] = '*';
+    hctx->path[dlen+1] = '\0';
+    WCHAR wbuf[4096];
+    int wlen = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS,
+                                   hctx->path, dlen+1,
+                                   wbuf, (sizeof(wbuf)/sizeof(*wbuf))-2);
+    if (0 == wlen) return -1;
+    hctx->hFind = FindFirstFileExW(wbuf, FindExInfoBasic, &hctx->ffd,
+                                   FindExSearchNameMatch, NULL,
+                                   FIND_FIRST_EX_LARGE_FETCH);
+    if (INVALID_HANDLE_VALUE == hctx->hFind) {
+        if (GetLastError() != ERROR_FILE_NOT_FOUND) {
+            log_error(r->conf.errh, __FILE__, __LINE__,
+              "FindFirstFileEx failed: %s", r->physical.path.ptr);
+            return -1;
+        }
+    }
+  #else
   #ifndef _ATFILE_SOURCE /*(not using fdopendir unless _ATFILE_SOURCE)*/
     hctx->dfd = -1;
     hctx->dp = opendir(hctx->path);
@@ -1016,6 +1050,7 @@ static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
         }
         return -1;
     }
+  #endif
 
     if (hctx->conf.json) return 0;
 
@@ -1030,14 +1065,35 @@ static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
 }
 
 static int http_read_directory(handler_ctx * const p) {
-	struct dirent *dent;
 	const int hide_dotfiles = p->conf.hide_dot_files;
 	const uint32_t name_max = p->name_max;
-	struct stat st;
+  #ifdef _WIN32
+	int count = 0;
+	if (INVALID_HANDLE_VALUE == p->hFind) {
+		/* GetLastError() == ERROR_FILE_NOT_FOUND
+		 * (other errors handled in http_open_directory()) */
+	}
+	else do
+  #else
 	int count = -1;
-	while (++count < DIRLIST_BATCH && (dent = readdir(p->dp)) != NULL) {
+	struct dirent *dent;
+	struct stat st;
+	while (++count < DIRLIST_BATCH && (dent = readdir(p->dp)) != NULL)
+  #endif
+	{
+	  #ifdef _WIN32
+		/* WC_ERR_INVALID_CHARS not used in string conversion since
+		 * expecting valid unicode from reading directory */
+		const char * const d_name = p->fnUTF8;
+		uint32_t dsz = (uint32_t)
+		  WideCharToMultiByte(CP_UTF8, 0, p->ffd.cFileName, -1,
+		                      p->fnUTF8, sizeof(p->fnUTF8), NULL, NULL);
+		if (0 == dsz) continue;
+		--dsz;
+	  #else
 		const char * const d_name = dent->d_name;
 		const uint32_t dsz = (uint32_t) _D_EXACT_NAMLEN(dent);
+	  #endif
 		if (d_name[0] == '.') {
 			if (hide_dotfiles)
 				continue;
@@ -1074,6 +1130,7 @@ static int http_read_directory(handler_ctx * const p) {
 		force_assert(dsz < sizeof(dent->d_name));
 	  #endif
 
+	  #ifndef _WIN32
 	  #ifndef _ATFILE_SOURCE
 		memcpy(p->path_file, d_name, dsz + 1);
 		if (stat(p->path, &st) != 0)
@@ -1082,6 +1139,7 @@ static int http_read_directory(handler_ctx * const p) {
 		/*(XXX: follow symlinks, like stat(); not using AT_SYMLINK_NOFOLLOW) */
 		if (0 != fstatat(p->dfd, d_name, &st, 0))
 			continue; /* file *just* disappeared? */
+	  #endif
 	  #endif
 
 		if (p->jb) { /* json output */
@@ -1095,7 +1153,12 @@ static int http_read_directory(handler_ctx * const p) {
 
 			const char *t;
 			size_t tlen;
-			if (!S_ISDIR(st.st_mode)) {
+		  #ifndef _WIN32
+			if (!S_ISDIR(st.st_mode))
+		  #else
+			if (!(p->ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
+		  #endif
+			{
 				t =           "\",\"type\":\"file\",\"size\":";
 				tlen = sizeof("\",\"type\":\"file\",\"size\":")-1;
 			}
@@ -1107,30 +1170,69 @@ static int http_read_directory(handler_ctx * const p) {
 			char mstr[LI_ITOSTRING_LENGTH];
 			struct const_iovec iov[] = {
 			  { t, tlen }
+			 #ifndef _WIN32
 			 ,{ sstr, li_itostrn(sstr, sizeof(sstr), st.st_size) }
 			 ,{ CONST_STR_LEN(",\"mtime\":") }
 			 ,{ mstr, li_itostrn(mstr, sizeof(mstr), TIME64_CAST(st.st_mtime)) }
+			 #else
+			 ,{ sstr, li_itostrn(sstr, sizeof(sstr),
+			                     ((int64_t)p->ffd.nFileSizeHigh << 32)
+			                             | p->ffd.nFileSizeLow) }
+			 ,{ CONST_STR_LEN(",\"mtime\":") }
+			 ,{ mstr, li_itostrn(mstr, sizeof(mstr),
+			            ((((int64_t)p->ffd.ftLastWriteTime.dwHighDateTime << 32)
+			                      | p->ffd.ftLastWriteTime.dwLowDateTime)
+			            / 10000000 - 11644473600LL)) }
+			 #endif
 			 ,{ CONST_STR_LEN("}") }
 			};
 			buffer_append_iovec(p->jb, iov, sizeof(iov)/sizeof(*iov));
 			continue;
 		}
 
+	  #ifndef _WIN32
 		dirls_list_t * const list = !S_ISDIR(st.st_mode) ? &p->files : &p->dirs;
+	  #else  /* _WIN32 */
+		dirls_list_t * const list =
+		  !(p->ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
+		    ? &p->files
+		    : &p->dirs;
+	  #endif /* _WIN32 */
 		if (!(list->used & (DIRLIST_BLOB_SIZE-1)))
 			ck_realloc_u32((void **)&list->ent, list->used,
 			               DIRLIST_BLOB_SIZE, sizeof(*list->ent));
 		dirls_entry_t * const tmp = list->ent[list->used++] =
 		  (dirls_entry_t*) ck_malloc(sizeof(dirls_entry_t) + 1 + dsz);
+	  #ifdef _WIN32 /*(convert 100ns ticks since 1 Jan 1601 to unix time_t)*/
+		/*(future: preserve FILETIME here and use Windows fn to format time
+		 * below instead of localtime_r() and buffer_append_strftime())*/
+		tmp->mtime = (time_t)
+                  ((((int64_t)p->ffd.ftLastWriteTime.dwHighDateTime << 32)
+		            | p->ffd.ftLastWriteTime.dwLowDateTime)
+                  / 10000000 - 11644473600LL);
+		tmp->size = ((int64_t)p->ffd.nFileSizeHigh << 32) | p->ffd.nFileSizeLow;
+	  #else
 		tmp->mtime = st.st_mtime;
 		tmp->size  = st.st_size;
+	  #endif
 		tmp->namelen = dsz;
 		memcpy(DIRLIST_ENT_NAME(tmp), d_name, dsz + 1);
 	}
+  #ifdef _WIN32
+	  while (++count < DIRLIST_BATCH && FindNextFileW(p->hFind, &p->ffd) != 0);
+	if (count == DIRLIST_BATCH)
+		return HANDLER_WAIT_FOR_EVENT;
+	if (GetLastError() != ERROR_NO_MORE_FILES) {
+		/*(some other GetLastError() value; ignore; truncate listing)*/
+	}
+	FindClose(p->hFind);
+	p->hFind = INVALID_HANDLE_VALUE;
+  #else
 	if (count == DIRLIST_BATCH)
 		return HANDLER_WAIT_FOR_EVENT;
 	closedir(p->dp);
 	p->dp = NULL;
+  #endif
 
 	return HANDLER_FINISHED;
 }
@@ -1165,7 +1267,11 @@ static void http_list_directory(request_st * const r, handler_ctx * const hctx)
 		buffer_append_string_len(out, CONST_STR_LEN("/\">"));
 		buffer_append_string_encoded(out, DIRLIST_ENT_NAME(tmp), tmp->namelen, ENCODING_MINIMAL_XML);
 		buffer_append_string_len(out, CONST_STR_LEN("</a>/</td><td class=\"m\">"));
+	  #ifdef __MINGW32__
+		buffer_append_strftime(out, "%Y-%b-%d %H:%M:%S", localtime64_r(&tmp->mtime, &tm));
+	  #else
 		buffer_append_strftime(out, "%Y-%b-%d %T", localtime64_r(&tmp->mtime, &tm));
+	  #endif
 		buffer_append_string_len(out, CONST_STR_LEN("</td><td class=\"s\">- &nbsp;</td><td class=\"t\">Directory</td></tr>\n"));
 
 		if (buffer_string_space(out) < 256) {
@@ -1206,7 +1312,11 @@ static void http_list_directory(request_st * const r, handler_ctx * const hctx)
 		buffer_append_string_len(out, CONST_STR_LEN("\">"));
 		buffer_append_string_encoded(out, DIRLIST_ENT_NAME(tmp), tmp->namelen, ENCODING_MINIMAL_XML);
 		buffer_append_string_len(out, CONST_STR_LEN("</a></td><td class=\"m\">"));
+	  #ifdef __MINGW32__
+		buffer_append_strftime(out, "%Y-%b-%d %H:%M:%S", localtime64_r(&tmp->mtime, &tm));
+	  #else
 		buffer_append_strftime(out, "%Y-%b-%d %T", localtime64_r(&tmp->mtime, &tm));
+	  #endif
 		size_t buflen =
 		  http_list_directory_sizefmt(sizebuf, sizeof(sizebuf), tmp->size);
 		struct const_iovec iov[] = {
@@ -1275,11 +1385,11 @@ static void mod_dirlisting_json_append (request_st * const r, handler_ctx * cons
     if (hctx->jfn) {
         if (__builtin_expect( (write_all(hctx->jfd, BUF_PTR_LEN(jb)) < 0), 0)) {
             /*(cleanup, cease caching if error occurs writing to cache file)*/
+            close(hctx->jfd);
+            hctx->jfd = -1;
             unlink(hctx->jfn);
             free(hctx->jfn);
             hctx->jfn = NULL;
-            close(hctx->jfd);
-            hctx->jfd = -1;
         }
         /* Note: writing cache file is separate from the response so that if an
          * error occurs with cache, the response still proceeds.  While this is
@@ -1548,8 +1658,6 @@ static int mod_dirlisting_write_cq (const int fd, chunkqueue * const cq, log_err
 }
 
 
-#include <sys/stat.h>   /* mkdir() */
-#include <sys/types.h>
 /*(similar to mod_deflate.c:mkdir_recursive(), but starts mid-path)*/
 static int mkdir_recursive (char *dir, size_t off) {
     char *p = dir+off;
@@ -1592,14 +1700,22 @@ static void mod_dirlisting_cache_add (request_st * const r, handler_ctx * const
     memcpy(oldpath, tb->ptr, len+7+1); /*(include '\0')*/
     const int fd = fdevent_mkostemp(oldpath, 0);
     if (fd < 0) return;
-    if (mod_dirlisting_write_cq(fd, &r->write_queue, r->conf.errh)
-        && 0 == fdevent_rename(oldpath, newpath)) {
+    int rc = mod_dirlisting_write_cq(fd, &r->write_queue, r->conf.errh);
+  #ifdef _WIN32
+    close(fd); /*(rename fails if file is open; MS filesystem limitation)*/
+  #endif
+    if (rc && 0 == fdevent_rename(oldpath, newpath)) {
         stat_cache_invalidate_entry(newpath, len);
         /* Cache-Control and ETag (also done in mod_dirlisting_cache_check())*/
         mod_dirlisting_cache_control(r, hctx->conf.cache->max_age);
         if (0 != r->conf.etag_flags) {
             struct stat st;
-            if (0 == fstat(fd, &st)) {
+          #ifdef _WIN32
+            if (0 == stat(newpath, &st))
+          #else
+            if (0 == fstat(fd, &st))
+          #endif
+            {
                 buffer * const vb =
                   http_header_response_set_ptr(r, HTTP_HEADER_ETAG,
                                                CONST_STR_LEN("ETag"));
@@ -1609,7 +1725,9 @@ static void mod_dirlisting_cache_add (request_st * const r, handler_ctx * const
     }
     else
         unlink(oldpath);
+  #ifndef _WIN32
     close(fd);
+  #endif
 }
 
 
@@ -1645,6 +1763,8 @@ static void mod_dirlisting_cache_json (request_st * const r, handler_ctx * const
     force_assert(len < PATH_MAX);
     memcpy(newpath, hctx->jfn, len);
     newpath[len] = '\0';
+    close(hctx->jfd);
+    hctx->jfd = -1;
     if (0 == fdevent_rename(hctx->jfn, newpath))
         stat_cache_invalidate_entry(newpath, len);
     else
@@ -1655,6 +1775,7 @@ static void mod_dirlisting_cache_json (request_st * const r, handler_ctx * const
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_dirlisting_plugin_init(plugin *p);
 int mod_dirlisting_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
